@@ -347,9 +347,12 @@ std::vector<CutlassGemmConfig> get_candidate_configs_sm100(
       candidate_configs.push_back(CutlassGemmConfig{
           CutlassTileConfigSM100::CtaShape256x128x128B, MainloopScheduleType::AUTO,
           EpilogueScheduleType::AUTO, ClusterShape::ClusterShape_2x1x1});
-      // candidate_configs.push_back(CutlassGemmConfig{CutlassTileConfigSM100::CtaShape128x256x128B,
-      //     MainloopScheduleType::AUTO, EpilogueScheduleType::AUTO,
-      //     ClusterShape::ClusterShape_1x1x1});
+      candidate_configs.push_back(CutlassGemmConfig{
+          CutlassTileConfigSM100::CtaShape128x256x128B, MainloopScheduleType::AUTO,
+          EpilogueScheduleType::AUTO, ClusterShape::ClusterShape_1x1x1});
+      candidate_configs.push_back(CutlassGemmConfig{
+          CutlassTileConfigSM100::CtaShape256x256x128B, MainloopScheduleType::AUTO,
+          EpilogueScheduleType::AUTO, ClusterShape::ClusterShape_2x1x1});
       candidate_configs.push_back(CutlassGemmConfig{
           CutlassTileConfigSM100::CtaShape128x256x128B, MainloopScheduleType::AUTO,
           EpilogueScheduleType::AUTO, ClusterShape::ClusterShape_1x2x1});
@@ -478,7 +481,7 @@ std::vector<CutlassGemmConfig> get_candidate_configs(
   if (sm >= 100 && sm < 120 && (config_type_param & CutlassGemmConfig::BLACKWELL)) {
     return get_candidate_configs_sm100(config_type_param);
   }
-  if (sm == 120 && (config_type_param & CutlassGemmConfig::BLACKWELL)) {
+  if (sm >= 120 && (config_type_param & CutlassGemmConfig::BLACKWELL)) {
     return get_candidate_configs_sm120(config_type_param);
   }
 
@@ -487,7 +490,7 @@ std::vector<CutlassGemmConfig> get_candidate_configs(
   std::vector<CutlassGemmConfig> candidate_configs;
 
   bool const int8_configs_only = config_type_param & CutlassGemmConfig::INT8_ONLY;
-  int const min_stages = int8_configs_only ? 3 : 2;
+  int const min_stages = (sm == 89) ? 3 : int8_configs_only ? 3 : 2;
   int const max_stages = int8_configs_only ? 6 : (sm >= 80 ? 4 : 2);
   for (auto const& tile_config : tiles) {
     for (int stages = min_stages; stages <= max_stages; ++stages) {
