@@ -18,8 +18,7 @@ import logging
 import os
 import platform
 import sys
-from dataclasses import dataclass
-from typing import List, Optional
+from typing import Any, Dict, List, Optional
 
 import torch
 from cuda import cuda
@@ -66,7 +65,6 @@ def create_tensor_from_cuda_memory(
 
     # Get element size in bytes
     element_size = torch.tensor([], dtype=dtype).element_size()
-    total_size_bytes = numel * element_size
 
     # Create DLPack capsule for contiguous memory (stride = element_size, num_segments = numel)
     capsule_wrapper = create_dlpack_capsule(
@@ -132,8 +130,9 @@ def alloc_and_copy_to_cuda(host_ptr_array: List[int]) -> Optional[int]:
 
 
 if IS_BUILDING_DOCS:
+    # Mock classes for building docs
 
-    class MpiComm:
+    class MpiComm:  # type: ignore[no-redef]
         @classmethod
         def set_mpi_comm(cls, new_comm):
             pass
@@ -141,7 +140,7 @@ if IS_BUILDING_DOCS:
         def __getattr__(self, name):
             return None
 
-    class MnnvlMemory:
+    class MnnvlMemory:  # type: ignore[no-redef]
         initialized: bool = False
 
         current_mem_offset: int = 0
@@ -159,8 +158,8 @@ if IS_BUILDING_DOCS:
 
         dev_id: int = None
 
-        allocated_map = {}
-        address_refcnt = {}
+        allocated_map: Dict[int, Any] = {}
+        address_refcnt: Dict[int, Any] = {}
 
         def __init__(self, mapping: Mapping, size: int):
             pass
@@ -211,7 +210,7 @@ else:
     import pynvml
     from mpi4py import MPI
 
-    class MpiComm:
+    class MpiComm:  # type: ignore[no-redef]
         _comm: MPI.Intracomm = MPI.COMM_WORLD
 
         @classmethod
@@ -221,7 +220,7 @@ else:
         def __getattr__(self, name):
             return getattr(self._comm, name)
 
-    class MnnvlMemory:
+    class MnnvlMemory:  # type: ignore[no-redef]
         initialized: bool = False
 
         current_mem_offset: int = 0
@@ -239,8 +238,8 @@ else:
 
         dev_id: int = None
 
-        allocated_map = {}
-        address_refcnt = {}
+        allocated_map: Dict[int, Any] = {}
+        address_refcnt: Dict[int, Any] = {}
 
         def __init__(self, mapping: Mapping, size: int):
             self.mapping = mapping
@@ -342,17 +341,17 @@ else:
             dev_id = int(dev)
             if MnnvlMemory.dev_id is None:
                 MnnvlMemory.dev_id = dev_id
-            assert (
-                dev_id == MnnvlMemory.dev_id
-            ), f"Different dev_id found dev_id={dev_id} but MnnvlMemory.dev_id={MnnvlMemory.dev_id}"
+            assert dev_id == MnnvlMemory.dev_id, (
+                f"Different dev_id found dev_id={dev_id} but MnnvlMemory.dev_id={MnnvlMemory.dev_id}"
+            )
             comm = MnnvlMemory.get_comm(mapping)
             comm_rank = comm.Get_rank()
             comm_size = comm.Get_size()
             all_rank_allocate_sizes = comm.allgather(size)
             assert len(all_rank_allocate_sizes) == comm_size
-            assert all(
-                x == size for x in all_rank_allocate_sizes
-            ), "Not all rank allocating same size."
+            assert all(x == size for x in all_rank_allocate_sizes), (
+                "Not all rank allocating same size."
+            )
             granularity = MnnvlMemory.get_allocation_granularity(dev_id)
             aligned_size = (size + granularity - 1) // granularity * granularity
 
@@ -496,7 +495,7 @@ else:
             # But it is not equivalent to MNNVL support.
             # May need better support check.
             arch = platform.machine().lower()
-            if not "aarch64" in arch:
+            if "aarch64" not in arch:
                 return False
             return MnnvlMemory.support_nvlink(True)
 
@@ -517,8 +516,6 @@ class McastDeviceMemory:
         primary_ctx = checkCudaErrors(cuda.cuDevicePrimaryCtxRetain(cu_device))
         checkCudaErrors(cuda.cuCtxSetCurrent(primary_ctx))
 
-        current_context = checkCudaErrors(cuda.cuCtxGetCurrent())
-
         # Set CUDA device
         import cuda.cudart as cudart
 
@@ -539,9 +536,9 @@ class McastDeviceMemory:
         self.signal_pads_dev = 0  # std::vector<CUdeviceptr> mSignalPadsDev
         self.uc_ptrs_dev = 0
         self.mc_handle = 0  # CUmemGenericAllocationHandle mMcHandle
-        self.uc_handles: List[int] = (
-            []
-        )  # std::vector<CUmemGenericAllocationHandle> mUcHandles
+        self.uc_handles: List[
+            int
+        ] = []  # std::vector<CUmemGenericAllocationHandle> mUcHandles
 
         # Signal pad constants
         self.SIGNAL_PAD_ALIGNMENT = 16
@@ -581,8 +578,6 @@ class McastDeviceMemory:
                     "[McastDeviceMemory] Device does not support fabric handle."
                 )
 
-            current_context = checkCudaErrors(cuda.cuCtxGetCurrent())
-
             self._alloc_mn_mcast_mem(buf_size)
         else:
             # For single-node NVLS, would need to implement _alloc_nvls_mcast_mem
@@ -684,7 +679,7 @@ class McastDeviceMemory:
     def get_unicast_ptr(self, rank: int) -> int:
         """Get the raw unicast pointer to a given rank"""
         if rank >= len(self.uc_ptrs):
-            raise ValueError(f"Rank {rank} out of range (0-{len(self.uc_ptrs)-1})")
+            raise ValueError(f"Rank {rank} out of range (0-{len(self.uc_ptrs) - 1})")
 
         data_ptr = self.uc_ptrs[rank]
         # Note: In C++, this would call tensorrt_llm::common::registerMcastDevMemBuffer
@@ -711,7 +706,6 @@ class McastDeviceMemory:
         # Verify CUDA context
         try:
             current_device = checkCudaErrors(cuda.cuCtxGetDevice())
-            current_context = checkCudaErrors(cuda.cuCtxGetCurrent())
 
             if int(current_device) != self.device_idx:
                 print(
