@@ -15,7 +15,7 @@ limitations under the License.
 """
 
 import os
-from typing import List, Optional
+from typing import List
 
 import jinja2
 import torch
@@ -879,12 +879,12 @@ def gen_batch_attention_module(
         use_profiler,
     )
 
-    additional_tensor_names = []
-    additional_tensor_dtypes = []
-    additional_scalar_names = []
-    additional_scalar_dtypes = []
+    additional_tensor_names: List[str] = []
+    additional_tensor_dtypes: List[str] = []
+    additional_scalar_names: List[str] = []
+    additional_scalar_dtypes: List[str] = []
     variant_name = f"StandardAttention<{str(use_logits_soft_cap).lower()}>"
-    variant_decl = f"#include<flashinfer/attention/variants.cuh>"
+    variant_decl = "#include<flashinfer/attention/variants.cuh>"
 
     return gen_customize_batch_attention_module(
         uri,
