@@ -3,8 +3,9 @@ import logging
 import os
 import re
 import warnings
+from contextlib import nullcontext
 from pathlib import Path
-from typing import List, Optional, Union
+from typing import List, Optional, Sequence, Union
 
 import torch
 import torch.utils.cpp_extension as torch_cpp_ext
@@ -31,15 +32,16 @@ class FlashInferJITLogger(logging.Logger):
         self.addHandler(logging.FileHandler(log_path))
         # set the format of the log
         self.handlers[0].setFormatter(
-            logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
+            logging.Formatter(
+                "%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - flashinfer.jit: %(message)s"
+            )
         )
         self.handlers[1].setFormatter(
-            logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
+            logging.Formatter(
+                "%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - flashinfer.jit: %(message)s"
+            )
         )
 
-    def info(self, msg):
-        super().info("flashinfer.jit: " + msg)
-
 
 logger = FlashInferJITLogger("flashinfer.jit")
 
@@ -87,7 +89,7 @@ class JitSpec:
         return jit_env.FLASHINFER_JIT_DIR / self.name / f"{self.name}.so"
 
     def get_library_path(self) -> Path:
-        if self.aot_path.exists():
+        if self.is_aot:
             return self.aot_path
         return self.jit_library_path
 
@@ -95,6 +97,14 @@ class JitSpec:
     def aot_path(self) -> Path:
         return jit_env.FLASHINFER_AOT_DIR / self.name / f"{self.name}.so"
 
+    @property
+    def is_aot(self) -> bool:
+        return self.aot_path.exists()
+
+    @property
+    def lock_path(self) -> Path:
+        return get_tmpdir() / f"{self.name}.lock"
+
     def write_ninja(self) -> None:
         ninja_path = self.ninja_path
         ninja_path.parent.mkdir(parents=True, exist_ok=True)
@@ -109,18 +119,14 @@ class JitSpec:
         )
         write_if_different(ninja_path, content)
 
-    def build(self, verbose: bool) -> None:
-        tmpdir = get_tmpdir()
-        with FileLock(tmpdir / f"{self.name}.lock", thread_local=False):
+    def build(self, verbose: bool, need_lock: bool = True) -> None:
+        lock = (
+            FileLock(self.lock_path, thread_local=False) if need_lock else nullcontext()
+        )
+        with lock:
             run_ninja(jit_env.FLASHINFER_JIT_DIR, self.ninja_path, verbose)
 
-    def build_and_load(self, class_name: str = None):
-        if self.aot_path.exists():
-            so_path = self.aot_path
-        else:
-            so_path = self.jit_library_path
-            verbose = os.environ.get("FLASHINFER_JIT_VERBOSE", "0") == "1"
-            self.build(verbose)
+    def load(self, so_path: Path, class_name: str = None):
         load_class = class_name is not None
         loader = torch.classes if load_class else torch.ops
         loader.load_library(so_path)
@@ -129,10 +135,24 @@ class JitSpec:
             return cls
         return getattr(loader, self.name)
 
+    def build_and_load(self, class_name: str = None):
+        if self.is_aot:
+            return self.load(self.aot_path, class_name)
+
+        # Guard both build and load with the same lock to avoid race condition
+        # where another process is building the library and removes the .so file.
+        with FileLock(self.lock_path, thread_local=False):
+            so_path = self.jit_library_path
+            verbose = os.environ.get("FLASHINFER_JIT_VERBOSE", "0") == "1"
+            self.build(verbose, need_lock=False)
+            result = self.load(so_path, class_name)
+
+        return result
+
 
 def gen_jit_spec(
     name: str,
-    sources: List[Union[str, Path]],
+    sources: Sequence[Union[str, Path]],
     extra_cflags: Optional[List[str]] = None,
     extra_cuda_cflags: Optional[List[str]] = None,
     extra_ldflags: Optional[List[str]] = None,
@@ -169,17 +189,18 @@ def gen_jit_spec(
         cflags += extra_cflags
     if extra_cuda_cflags is not None:
         cuda_cflags += extra_cuda_cflags
-    if extra_include_paths is not None:
-        extra_include_paths = [Path(x) for x in extra_include_paths]
-    sources = [Path(x) for x in sources]
 
     spec = JitSpec(
         name=name,
-        sources=sources,
+        sources=[Path(x) for x in sources],
         extra_cflags=cflags,
         extra_cuda_cflags=cuda_cflags,
         extra_ldflags=extra_ldflags,
-        extra_include_dirs=extra_include_paths,
+        extra_include_dirs=(
+            [Path(x) for x in extra_include_paths]
+            if extra_include_paths is not None
+            else None
+        ),
         needs_device_linking=needs_device_linking,
     )
     spec.write_ninja()
