@@ -15,13 +15,10 @@ limitations under the License.
 """
 
 import ctypes
-import functools
 import hashlib
 import os
 import shutil
-import sys
 import time
-from typing import Any, Callable, Dict, List, Literal, Optional, Tuple, Union, overload
 
 import filelock
 
@@ -52,7 +49,7 @@ def download_file(source, local_path, retries=3, delay=5, timeout=10, lock_timeo
     - bool: True if download or copy is successful, False otherwise.
     """
 
-    import requests
+    import requests  # type: ignore[import-untyped]
 
     lock_path = f"{local_path}.lock"  # Lock file path
     lock = filelock.FileLock(lock_path, timeout=lock_timeout)
@@ -110,7 +107,7 @@ def download_file(source, local_path, retries=3, delay=5, timeout=10, lock_timeo
             logger.info(f"Lock file {lock_path} removed.")
 
 
-def load_cubin(cubin_path, sha256):
+def load_cubin(cubin_path, sha256) -> bytes:
     """
     Load a cubin from the provide local path and
     ensure that the sha256 signature matches.
@@ -131,13 +128,13 @@ def load_cubin(cubin_path, sha256):
             logger.warning(
                 f"sha256 mismatch (expected {sha256} actual {actual_sha}) for {cubin_path}"
             )
-    except:
-        pass
+    except Exception as e:
+        print(f"Failed to load cubin at {cubin_path} with sha256={sha256}: {e}")
     logger.info(f"Failed loading {cubin_path}")
-    return ""
+    return b""
 
 
-def get_cubin(name, sha256):
+def get_cubin(name, sha256, file_extension=".cubin"):
     """
     Load a cubin from the local cache directory with {name} and
     ensure that the sha256 signature matches.
@@ -147,7 +144,7 @@ def get_cubin(name, sha256):
     Returns:
     None on failure.
     """
-    cubin_fname = name + ".cubin"
+    cubin_fname = name + file_extension
     cubin_path = FLASHINFER_CACHE_DIR / "cubins" / cubin_fname
     cubin = load_cubin(cubin_path, sha256)
     if cubin:
@@ -188,3 +185,32 @@ def setup_cubin_loader(dll_path: str):
     dll_cubin_handlers[dll_path] = cb
 
     _LIB.FlashInferSetCubinCallback(cb)
+
+
+dll_metainfo_handlers = {}
+
+
+def setup_metainfo_loader(dll_path: str):
+    if dll_path in dll_metainfo_handlers:
+        return
+
+    _LIB = ctypes.CDLL(dll_path)
+
+    # Define the correct callback type
+    CALLBACK_TYPE = ctypes.CFUNCTYPE(
+        None, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p
+    )
+
+    def get_metainfo_callback(name, sha256, extension):
+        metainfo = get_cubin(
+            name.decode("utf-8"), sha256.decode("utf-8"), extension.decode("utf-8")
+        )
+        _LIB.FlashInferSetCurrentMetaInfo(
+            convert_to_ctypes_char_p(metainfo), ctypes.c_int(len(metainfo))
+        )
+
+    # Create the callback and keep a reference to prevent GC
+    cb = CALLBACK_TYPE(get_metainfo_callback)
+    dll_metainfo_handlers[dll_path] = cb
+
+    _LIB.FlashInferSetMetaInfoCallback(cb)
