@@ -15,14 +15,14 @@ def check_dim(d, x: torch.Tensor):
 def check_shape(a: torch.Tensor, b: torch.Tensor):
     assert a.dim() == b.dim(), "tensors should have same dim"
     for i in range(a.dim()):
-        assert a.size(i) == b.size(
-            i
-        ), f"tensors shape mismatch, {a.size()} and {b.size()}"
+        assert a.size(i) == b.size(i), (
+            f"tensors shape mismatch, {a.size()} and {b.size()}"
+        )
 
 
 def check_device(tensors: List[torch.Tensor]):
     device = tensors[0].device
     for t in tensors:
-        assert (
-            t.device == device
-        ), f"All tensors should be on the same device, but got {device} and {t.device}"
+        assert t.device == device, (
+            f"All tensors should be on the same device, but got {device} and {t.device}"
+        )
