@@ -16,18 +16,18 @@
 
 #pragma once
 
+#include <string>
+
 #include "DevKernel.h"
 #include "RoutingKernel.h"
 // #include "flashinfer/trtllm/common/cudaDriverWrapper.h"
-#include <string>
-
 #include "flashinfer/trtllm/batched_gemm/KernelRunner.h"
 #include "flashinfer/trtllm/batched_gemm/trtllmGen_bmm_export/trtllm/gen/DtypeDecl.h"
 #include "flashinfer/trtllm/common/cudaUtils.h"
 
 namespace tensorrt_llm {
 namespace kernels {
-namespace trtllmGenFp8BlockScaleMoe {
+namespace trtllmgen_moe {
 
 namespace Routing {
 
@@ -120,22 +120,37 @@ class Runner {
 namespace PermuteGemm1 {
 class Runner {
  public:
-  explicit Runner(batchedGemm::trtllm::gen::Dtype dtypeElt, bool useDeepSeekFp8, int tileTokensDim);
+  explicit Runner(batchedGemm::trtllm::gen::Dtype dtypeAct,
+                  batchedGemm::trtllm::gen::Dtype dtypeWeights, bool useDeepSeekFp8,
+                  int tileTokensDim, ActType actType, bool useShuffledMatrixA,
+                  batchedGemm::gemm::MatrixLayout weight_layout);
 
   size_t getWorkspaceSizeInBytes(int32_t topK, int32_t hiddenSize, int32_t intermediateSize,
-                                 int32_t numExperts, int32_t numTokens);
+                                 int32_t numExperts, int32_t numTokens, int32_t configIndex) const;
+
+  [[nodiscard]] int32_t getDefaultValidConfigIndex(int32_t topK, int32_t hiddenSize,
+                                                   int32_t intermediateSize, int32_t numExperts,
+                                                   int32_t numTokens) const;
+
+  [[nodiscard]] bool isValidConfigIndex(int32_t configIndex, int32_t topK, int32_t hiddenSize,
+                                        int32_t intermediateSize, int32_t numExperts,
+                                        int32_t numTokens) const;
+
+  [[nodiscard]] std::vector<int64_t> getPassingConfigIndices() const;
 
   void run(void* hiddenState, void* hiddenStateScale, void* weight, void* weightScale,
            void* expertWeights, float* outputScalesScalar, float* outputScalesGateScalar,
+           float* ptrBias, float* ptrSwiGluAlpha, float* ptrSwiGluBeta, float* ptrClampLimit,
            void* output, void* outputScale, int32_t topK, int32_t hiddenSize,
            int32_t intermediateSize, int32_t numExperts, int32_t numTokens,
            int32_t* permutedIdxToTokenIdx, int32_t* ptrNumNonExitingCtas,
            int32_t* ptrTotalNumPaddedTokens, int32_t* ptrCtaIdxXyToBatchIdx,
            int32_t* ptrCtaIdxXyToMnLimit, void* bmm1Workspace, bool useRoutingScalesOnInput,
-           int device, cudaStream_t stream);
+           int device, cudaStream_t stream, int32_t configIndex);
 
  private:
-  batchedGemm::trtllm::gen::Dtype mDtypeElt;
+  batchedGemm::trtllm::gen::Dtype mDtypeAct;
+  batchedGemm::trtllm::gen::Dtype mDtypeWeights;
   int32_t mTileTokensDim;
   tensorrt_llm::kernels::TrtllmGenBatchedGemmRunner mRunner;
 };
@@ -144,23 +159,37 @@ class Runner {
 namespace Gemm2 {
 class Runner {
  public:
-  explicit Runner(batchedGemm::trtllm::gen::Dtype dtypeElt,
+  explicit Runner(batchedGemm::trtllm::gen::Dtype dtypeAct,
+                  batchedGemm::trtllm::gen::Dtype dtypeWeights,
                   batchedGemm::trtllm::gen::Dtype outputDtype, bool useDeepSeekFp8,
-                  int tileTokensDim);
+                  int tileTokensDim, bool useShuffledMatrixA,
+                  batchedGemm::gemm::MatrixLayout weight_layout);
 
   size_t getWorkspaceSizeInBytes(int32_t topK, int32_t hiddenSize, int32_t intermediateSize,
-                                 int32_t numExperts, int32_t numTokens);
+                                 int32_t numExperts, int32_t numTokens, int32_t configIndex) const;
+
+  [[nodiscard]] int32_t getDefaultValidConfigIndex(int32_t topK, int32_t hiddenSize,
+                                                   int32_t intermediateSize, int32_t numExperts,
+                                                   int32_t numTokens) const;
+
+  [[nodiscard]] bool isValidConfigIndex(int32_t configIndex, int32_t topK, int32_t hiddenSize,
+                                        int32_t intermediateSize, int32_t numExperts,
+                                        int32_t numTokens) const;
+
+  [[nodiscard]] std::vector<int64_t> getPassingConfigIndices() const;
 
   void run(void* permutedHiddenState, void* permutedHiddenStateScale, void* weight,
-           void* weightScale, float* outputScalesScalar, void* output, void* outputScale,
-           int32_t topK, int32_t hiddenSize, int32_t intermediateSize, int32_t numExperts,
-           int32_t numTokens, int32_t* ptrNumNonExitingCtas, int32_t* ptrTotalNumPaddedTokens,
-           int32_t* ptrCtaIdxXyToBatchIdx, int32_t* ptrCtaIdxXyToMnLimit, void* bmm2Workspace,
-           int device, cudaStream_t stream);
+           void* weightScale, float* outputScalesScalar, float* ptrBias, void* output,
+           void* outputScale, int32_t topK, int32_t hiddenSize, int32_t intermediateSize,
+           int32_t numExperts, int32_t numTokens, int32_t* ptrNumNonExitingCtas,
+           int32_t* ptrTotalNumPaddedTokens, int32_t* ptrCtaIdxXyToBatchIdx,
+           int32_t* ptrCtaIdxXyToMnLimit, void* bmm2Workspace, int device, cudaStream_t stream,
+           int32_t configIndex);
 
  private:
-  batchedGemm::trtllm::gen::Dtype mDtypeElt;
-  batchedGemm::trtllm::gen::Dtype mOutputDtype;
+  batchedGemm::trtllm::gen::Dtype mDtypeAct;
+  batchedGemm::trtllm::gen::Dtype mDtypeWeights;
+  batchedGemm::trtllm::gen::Dtype mDtypeOut;
   int32_t mTileTokensDim;
   tensorrt_llm::kernels::TrtllmGenBatchedGemmRunner mRunner;
 };
@@ -184,9 +213,19 @@ struct MoERunnerArgs {
   void* gemm2_weights = nullptr;
   void* gemm2_weights_scale = nullptr;
 
+  float* gemm1_bias = nullptr;
+  float* gemm1_alpha = nullptr;
+  float* gemm1_beta = nullptr;
+  float* gemm1_clamp_limit = nullptr;
+  float* gemm2_bias = nullptr;
+
   int32_t num_tokens{0};
   int32_t num_experts{0};
+  // Hidden dimension input of MoE block. It might be padded.
   int32_t hidden_size{0};
+  // Hidden dimension output of MoE block. It is not padded.
+  // If not provided it is the same as hidden_size.
+  std::optional<int32_t> hidden_size_output;
   // TODO: only compiled routing kernel supports top_k = 8
   int32_t top_k{0};
   int32_t n_group{0};
@@ -211,6 +250,9 @@ struct MoERunnerArgs {
   // Output:
   void* output = nullptr;
   float* output_scale = nullptr;
+
+  // finalize
+  bool do_finalize{true};
 };
 
 struct MoEWorkspace {
@@ -259,15 +301,38 @@ struct MoEWorkspace {
   void* bmm2_workspace = nullptr;
 };
 
+// Config indices to be used with Batched GEMM runners
+struct MoEConfig {
+  int64_t gemm1Config;
+  int64_t gemm2Config;
+};
+
 class Runner {
  public:
   // FIXME: tileTokensDim is hardcoded for now
-  Runner(batchedGemm::trtllm::gen::Dtype dtypeElt, bool useDeepSeekFp8, int tileTokensDim = 8);
+  Runner(batchedGemm::trtllm::gen::Dtype dtypeAct, batchedGemm::trtllm::gen::Dtype dtypeWeights,
+         bool useDeepSeekFp8, int tileTokensDim = 8, ActType actType = ActType::SwiGlu,
+         bool useShuffledMatrixA = false,
+         batchedGemm::gemm::MatrixLayout weight_layout = batchedGemm::gemm::MatrixLayout::MajorK);
+  Runner(batchedGemm::trtllm::gen::Dtype dtypeElt, bool useDeepSeekFp8, int tileTokensDim = 8,
+         bool useShuffledMatrixA = false,
+         batchedGemm::gemm::MatrixLayout weight_layout = batchedGemm::gemm::MatrixLayout::MajorK);
 
   void run(MoERunnerArgs const& args, MoEWorkspace const& workspace, int device,
-           cudaStream_t stream);
+           cudaStream_t stream, int64_t configIndex);
 
-  std::tuple<int32_t, int32_t> getWorkspaceSizeInBytes(MoERunnerArgs const& args);
+  [[nodiscard]] std::tuple<int32_t, int32_t> getWorkspaceSizeInBytes(MoERunnerArgs const& args,
+                                                                     int64_t configIndex) const;
+
+  [[nodiscard]] std::vector<int64_t> getValidConfigIndices(int32_t topK, int32_t hiddenSize,
+                                                           int32_t intermediateSize,
+                                                           int32_t numLocalExperts,
+                                                           int32_t numTokens) const;
+
+  [[nodiscard]] int64_t getDefaultValidConfigIndex(int32_t topK, int32_t hiddenSize,
+                                                   int32_t intermediateSize,
+                                                   int32_t numLocalExperts,
+                                                   int32_t numTokens) const;
 
  private:
   void setOpsData(MoERunnerArgs const& args, MoEWorkspace const& workspace,
@@ -278,9 +343,14 @@ class Runner {
  private:
   PermuteGemm1::Runner mPermuteGemm1;
   Gemm2::Runner mGemm2;
+
+  // This will be the cartesian product of the passing configs for gemm1 and gemm2
+  // This allows us to autotune the MoE as one operation instead of tuning gemm1 and gemm2
+  // separately
+  std::vector<MoEConfig> mPassingConfigs;
 };
 }  // namespace MoE
 
-}  // namespace trtllmGenFp8BlockScaleMoe
+}  // namespace trtllmgen_moe
 }  // namespace kernels
 }  // namespace tensorrt_llm
