@@ -2,7 +2,7 @@ import pytest
 import torch
 import torch.nn.functional as F
 
-from flashinfer import bmm_fp8
+from flashinfer import autotune, bmm_fp8
 
 
 def to_float8(x, dtype=torch.float8_e4m3fn):
@@ -21,10 +21,16 @@ def to_float8(x, dtype=torch.float8_e4m3fn):
 @pytest.mark.parametrize("input_dtype", [torch.float8_e4m3fn, torch.float8_e5m2])
 @pytest.mark.parametrize("mat2_dtype", [torch.float8_e4m3fn, torch.float8_e5m2])
 @pytest.mark.parametrize("res_dtype", [torch.bfloat16, torch.float16])
-@pytest.mark.parametrize("backend", ["cudnn", "cublas"])
-def test_bmm_fp8(b, m, n, k, input_dtype, mat2_dtype, res_dtype, backend):
+@pytest.mark.parametrize("backend", ["cudnn", "cublas", "cutlass"])
+@pytest.mark.parametrize("auto_tuning", [True, False])
+def test_bmm_fp8(b, m, n, k, input_dtype, mat2_dtype, res_dtype, backend, auto_tuning):
     if input_dtype == torch.float8_e5m2 and mat2_dtype == torch.float8_e5m2:
         pytest.skip("Invalid combination: both input and mat2 are e5m2")
+    if input_dtype == torch.float8_e5m2 or mat2_dtype == torch.float8_e5m2:
+        if backend == "cutlass":
+            pytest.skip("Invalid combination: cutlass does not support e5m2")
+    if auto_tuning and backend != "cutlass":
+        pytest.skip("Invalid combination: auto_tuning only supported for cutlass")
 
     input = torch.randn([b, m, k], device="cuda", dtype=torch.bfloat16)
     input_fp8, input_inv_s = to_float8(input, dtype=input_dtype)
@@ -35,9 +41,17 @@ def test_bmm_fp8(b, m, n, k, input_dtype, mat2_dtype, res_dtype, backend):
     reference = torch.bmm(input, mat2)
 
     res = torch.empty([b, m, n], device="cuda", dtype=res_dtype)
-    bmm_fp8(
-        input_fp8, mat2_fp8, input_inv_s, mat2_inv_s, res_dtype, res, backend=backend
-    )
+
+    with autotune(auto_tuning):
+        bmm_fp8(
+            input_fp8,
+            mat2_fp8,
+            input_inv_s,
+            mat2_inv_s,
+            res_dtype,
+            res,
+            backend=backend,
+        )
 
     cos_sim = F.cosine_similarity(reference.reshape(-1), res.reshape(-1), dim=0)
     assert cos_sim > 0.99
