@@ -1,6 +1,3 @@
-import math
-
-import numpy as np
 import pytest
 import torch
 
