@@ -53,13 +53,13 @@ def swizzle_blockscale(
     Returns:
         torch.Tensor: Swizzled tensor with the same shape as input.
     """
-    assert (
-        unswizzled_sf.dtype == torch.uint8
-    ), f"Input dtype must be uint8, got {unswizzled_sf.dtype}"
+    assert unswizzled_sf.dtype == torch.uint8, (
+        f"Input dtype must be uint8, got {unswizzled_sf.dtype}"
+    )
     assert unswizzled_sf.ndim == 3, f"Input must be 3D, got {unswizzled_sf.ndim}"
-    assert (
-        unswizzled_sf.shape[0] == b
-    ), f"Batch dimension must equal b, got {unswizzled_sf.shape[0]} != {b}"
+    assert unswizzled_sf.shape[0] == b, (
+        f"Batch dimension must equal b, got {unswizzled_sf.shape[0]} != {b}"
+    )
     padded_input_sf_chunked = [
         _pad_scale_factors(unswizzled_sf[i], m, n, sf_vec_size) for i in range(b)
     ]
